@@ -1,7 +1,7 @@
 `getRow` <- 
 function(noflevels, row.no, zerobased=FALSE) {
     max.combs <- prod(noflevels)
-    if (row.no > max.combs - zerobased) {
+    if (any(row.no > (max.combs - zerobased))) {
         cat("\n")
         stop("There cannot be more than ", max.combs, " rows.\n\n", call. = FALSE)
         }
@@ -9,6 +9,8 @@ function(noflevels, row.no, zerobased=FALSE) {
     if (!zerobased) {row.no <- row.no - 1}
     
     mbase <- c(rev(cumprod(rev(noflevels))), 1)[-1]
-    sapply(seq_len(noflevels), function(x) floor(row.no/mbase[x])%%noflevels[x])
+    t(sapply(row.no, function(x) {
+        sapply(seq(length(noflevels)), function(y) floor(x/mbase[y])%%noflevels[y])
+        }))
     }
 
